@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The node API of Hyper Estraier
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2007 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -52,10 +52,14 @@ void est_free_net_env(void);
  *************************************************************************************************/
 
 
-typedef struct {                         /* type of structure for a document in result */
+typedef struct _ESTRESDOC {              /* type of structure for a document in result */
   const char *uri;                       /* URI */
   CBMAP *attrs;                          /* attributes */
   char *snippet;                         /* snippet */
+  CBMAP *kwords;                         /* keywords */
+  double similarity;                     /* similarity to the parent document */
+  struct _ESTRESDOC **children;          /* child documents */
+  int cnum;                              /* number of child documents */
 } ESTRESDOC;
 
 typedef struct {                         /* type of structure for search result */
@@ -63,7 +67,10 @@ typedef struct {                         /* type of structure for search result
   int top;                               /* offset number of the top element */
   int max;                               /* maximum number of allcated elements */
   int dnum;                              /* number of effective elements */
+  ESTRESDOC **survivors;                 /* pointers to surviving documents */
+  int snum;                              /* number of surviving elements */
   CBMAP *hints;                          /* map object for hints */
+  time_t mdate;                          /* modification date */
 } ESTNODERES;
 
 
@@ -75,11 +82,20 @@ void est_noderes_delete(ESTNODERES *nres);
 /* Get a map object for hints of a node result object.
    `nres' specifies a node result object.
    The return value is a map object for hints.  Keys of the map are "VERSION", "NODE", "HIT",
-   "HINT#n", "DOCNUM", "WORDNUM", "TIME", "LINK#n", and "VIEW".  The life duration of the
-   returned object is synchronous with the one of the node result object. */
+   "HINT#n", "DOCNUM", "WORDNUM", "TIME", "TIME#n", "LINK#n", and "VIEW".  The life duration of
+   the returned object is synchronous with the one of the node result object. */
 CBMAP *est_noderes_hints(ESTNODERES *nres);
 
 
+/* Eclipse similar documents of a node result object.
+   `nres' specifies a node result object.
+   `num' specifies the number of documents to be shown.  If it is not more than 0, eclipse is
+   undone.
+   `limit' specifies the lower limit of similarity for documents to be eclipsed.  Similarity is
+   between 0.0 and 1.0. */
+void est_noderes_eclipse(ESTNODERES *nres, int num, double limit);
+
+
 /* Get the number of documents in a node result object.
    `nres' specifies a node result object.
    The return value is the number of documents in a node result object. */
@@ -120,14 +136,38 @@ const char *est_resdoc_attr(ESTRESDOC *rdoc, const char *name);
 
 /* Get the snippet of a result document object.
    `rdoc' specifies a result document object.
-   The return value is the snippet of the result document object.  There are tab separated
-   values.  Each line is a string to be shown.  Though most lines have only one field, some
-   lines have two fields.  If the second field exists, the first field is to be shown with
+   The return value is a string of the snippet of the result document object.  There are tab
+   separated values.  Each line is a string to be shown.  Though most lines have only one field,
+   some lines have two fields.  If the second field exists, the first field is to be shown with
    highlighted, and the second field means its normalized form.  The life duration of the
    returned string is synchronous with the one of the result document object. */
 const char *est_resdoc_snippet(ESTRESDOC *rdoc);
 
 
+/* Get keywords of a result document object.
+   `rdoc' specifies a result document object.
+   The return value is a string of serialized keywords of the result document object.  There are
+   tab separated values.  Keywords and their scores come alternately.  The life duration of the
+   returned string is synchronous with the one of the result document object. */
+const char *est_resdoc_keywords(ESTRESDOC *rdoc);
+
+
+/* Get an array of documents eclipsed by a result document object.
+   `rdoc' specifies a result document object.
+   `np' specifies the pointer to a variable to which the number of elements of the return value
+   is assigned.
+   The return value is an array of eclipsed result document objects.  The life duration of the
+   returned array and its elements is synchronous with the one of the result document object. */
+ESTRESDOC **est_resdoc_shadows(ESTRESDOC *rdoc, int *np);
+
+
+/* Get similarity of an eclipsed result document object.
+   `rdoc' specifies a result document object.
+   The return value is similarity of the result document object to the front document or -1.0
+   if it is not eclipsed. */
+double est_resdoc_similarity(ESTRESDOC *rdoc);
+
+
 
 /*************************************************************************************************
  * API for node
@@ -145,6 +185,14 @@ typedef struct {                         /* type of structure for a node object
   int dnum;                              /* number of documents */
   int wnum;                              /* number of words */
   double size;                           /* size of the database */
+  CBLIST *admins;                        /* list of names of administrators */
+  CBLIST *users;                         /* list of names of users */
+  CBLIST *links;                         /* list of expressions of linking nodes */
+  int wwidth;                            /* whole width of a snippet */
+  int hwidth;                            /* head width of a snippet */
+  int awidth;                            /* around width of a snippet */
+  CBLIST *ilist;                         /* list of the document iterator */
+  char *iprev;                           /* previous URI of the document iterator */
   int status;                            /* last status code */
   CBMAP *heads;                          /* extention headers */
 } ESTNODE;
@@ -188,6 +236,18 @@ void est_node_set_auth(ESTNODE *node, const char *name, const char *passwd);
 int est_node_status(ESTNODE *node);
 
 
+/* Synchronize updating contents of the database of a node.
+   `node' specifies a node connection object.
+   The return value is true if success, else it is false. */
+int est_node_sync(ESTNODE *node);
+
+
+/* Optimize the database of a node.
+   `node' specifies a node connection object.
+   The return value is true if success, else it is false. */
+int est_node_optimize(ESTNODE *node);
+
+
 /* Add a document to a node.
    `node' specifies a node connection object.
    `doc' specifies a document object.  The document object should have the URI attribute.
@@ -211,6 +271,14 @@ int est_node_out_doc(ESTNODE *node, int id);
 int est_node_out_doc_by_uri(ESTNODE *node, const char *uri);
 
 
+/* Edit attributes of a document in a node.
+   `node' specifies a node connection object.
+   `doc' specifies a document object.
+   The return value is true if success, else it is false.
+   Neither the ID nor the URI can not be changed. */
+int est_node_edit_doc(ESTNODE *node, ESTDOC *doc);
+
+
 /* Retrieve a document in a node.
    `node' specifies a node connection object.
    `id' specifies the ID number of a registered document.
@@ -247,6 +315,24 @@ char *est_node_get_doc_attr(ESTNODE *node, int id, const char *name);
 char *est_node_get_doc_attr_by_uri(ESTNODE *node, const char *uri, const char *name);
 
 
+/* Extract keywords of a document.
+   `node' specifies a node connection object.
+   `id' specifies the ID number of a registered document.
+   The return value is a new map object of keywords and their scores in decimal string or `NULL'
+   on error.  Because the object of the return value is opened with the function `cbmapopen', it
+   should be closed with the function `cbmapclose' if it is no longer in use. */
+CBMAP *est_node_etch_doc(ESTNODE *node, int id);
+
+
+/* Extract keywords of a document specified by URI in a node.
+   `node' specifies a node connection object.
+   `uri' specifies the URI of a registered document.
+   The return value is a new map object of keywords and their scores in decimal string or `NULL'
+   on error.  Because the object of the return value is opened with the function `cbmapopen', it
+   should be closed with the function `cbmapclose' if it is no longer in use. */
+CBMAP *est_node_etch_doc_by_uri(ESTNODE *node, const char *uri);
+
+
 /* Get the ID of a document spacified by URI.
    `node' specifies a node connection object.
    `uri' specifies the URI of a registered document.
@@ -286,7 +372,35 @@ int est_node_word_num(ESTNODE *node);
 double est_node_size(ESTNODE *node);
 
 
-/* Search documents corresponding a condition for a node.
+/* Get the usage ratio of the cache of a node.
+   `node' specifies a node connection object.
+   The return value is the usage ratio of the cache of the node.  On error, -1.0 is returned. */
+double est_node_cache_usage(ESTNODE *node);
+
+
+/* Get a list of names of administrators of a node.
+   `node' specifies a node connection object.
+   The return value is a list object of names of administrators.  On error, `NULL' is returned.
+   The life duration of the returned object is synchronous with the one of the node object. */
+const CBLIST *est_node_admins(ESTNODE *node);
+
+
+/* Get a list of names of users of a node.
+   `node' specifies a node connection object.
+   The return value is a list object of names of users.  On error, `NULL' is returned.  The life
+   duration of the returned object is synchronous with the one of the node object. */
+const CBLIST *est_node_users(ESTNODE *node);
+
+
+/* Get a list of expressions of links of a node.
+   `node' specifies a node connection object.
+   The return value is a list object of expressions of links.  Each element is a TSV string and
+   has three fields of the URL, the label, and the score.  On error, `NULL' is returned.  The
+   life duration of the returned object is synchronous with the one of the node object. */
+const CBLIST *est_node_links(ESTNODE *node);
+
+
+/* Search a node for documents corresponding a condition.
    `node' specifies a node connection object.
    `cond' specifies a condition object.
    `depth' specifies the depth of meta search.
@@ -295,11 +409,22 @@ double est_node_size(ESTNODE *node);
 ESTNODERES *est_node_search(ESTNODE *node, ESTCOND *cond, int depth);
 
 
+/* Set width of snippet in the result from a node.
+   `node' specifies a node connection object.
+   `wwidth' specifies whole width of a snippet.  By default, it is 480.  If it is 0, no snippet
+   is sent. If it is negative, whole body text is sent instead of snippet.
+   `hwidth' specifies width of strings picked up from the beginning of the text.  By default, it
+   is 96.  If it is negative 0, the current setting is not changed.
+   `awidth' specifies width of strings picked up around each highlighted word. By default, it is
+   96.  If it is negative, the current setting is not changed. */
+void est_node_set_snippet_width(ESTNODE *node, int wwidth, int hwidth, int awidth);
+
+
 /* Manage a user account of a node.
    `node' specifies a node connection object.
    `name' specifies the name of a user.
    `mode' specifies the operation mode.  0 means to delete the account.  1 means to set the
-   account as an administrator.  2 means to set the account as a normal user.
+   account as an administrator.  2 means to set the account as a guest.
    The return value is true if success, else it is false. */
 int est_node_set_user(ESTNODE *node, const char *name, int mode);
 
@@ -322,7 +447,8 @@ int est_node_set_link(ESTNODE *node, const char *url, const char *label, int cre
 #define ESTAGENTNAME   "HyperEstraier"   /* name of the user agent */
 #define ESTFORMTYPE    "application/x-www-form-urlencoded"  /* media type of docuemnt draft */
 #define ESTINFORMTYPE  "text/x-estraier-nodeinfo"  /* media type of node information */
-#define ESTRESULTTYPE  "text/x-estraier-result"    /* media type of search result */
+#define ESTSEARCHTYPE  "text/x-estraier-search"    /* media type of search result */
+#define ESTLISTTYPE    "text/x-estraier-list"      /* media type of list result */
 #define ESTDRAFTTYPE   "text/x-estraier-draft"     /* media type of docuemnt draft */
 #define ESTHTHVIA      "X-Estraier-Via"  /* header to escape from looping route */
 
@@ -404,20 +530,13 @@ void est_sock_recv_void(int sock);
 void est_sock_send_all(int sock, const char *buf, int len);
 
 
-/* Perform formatted output into a datum object.
-   `format' specifies a printf-like format string.
-   The conversion  character `%' can be used with such flag characters as `s', `d', `o', `u',
-   `x', `X', `c', `e', `E', `f', `g', `G', `@', `?', `%'.  `@' works as with `s' but escapes
-   meta characters of XML.  `?' works as with `s' but escapes meta characters of URL.  The other
-   conversion character work as with each original. */
-void est_datum_printf(CBDATUM *datum, const char *format, ...);
-
-
 /* Perform an interaction of a URL.
    `url' specifies a URL.
    `pxhost' specifies the host name of a proxy.  If it is `NULL', it is not used.
    `pxport' specifies the port number of the proxy.
    `outsec' specifies timeout in seconds.  If it is negative, it is not used.
+   `limsize' specifies limitation of the entity body of response.  If it is negative, it is not
+   used.
    `auth' specifies an authority information in such form as "user:pass".  If it is `NULL', it is
    not used.
    `reqheads' specifies a list object of extension headers.  If it is `NULL' it is not used.
@@ -433,13 +552,28 @@ void est_datum_printf(CBDATUM *datum, const char *format, ...);
    `resbody' specifies a datum object into which the entity body of response is stored.  If it is
    `NULL', it is not used.
    The return value is true if success, else it is false.
-   Headers of "Host", "Connection", "User-Agent", "Authorization", and "Content-Length" are sent
-   implicitly. */
-int est_url_shuttle(const char *url, const char *pxhost, int pxport, int outsec,
+   Headers of "Host", "Connection", "User-Agent", "Authorization", "Accept-Encoding", and
+   "Content-Length" are sent implicitly. */
+int est_url_shuttle(const char *url, const char *pxhost, int pxport, int outsec, int limsize,
                     const char *auth, const CBLIST *reqheads, const char *reqbody, int rbsiz,
                     int *rescodep, CBMAP *resheads, CBDATUM *resbody);
 
 
+/* Initialize the document iterator of a node.
+   `node' specifies a node connection object.
+   The return value is true if success, else it is false. */
+int est_node_iter_init(ESTNODE *node);
+
+
+/* Get the next document of the document iterator of a node.
+   `node' specifies a node connection object.
+   The return value is a document object though it does not have body text.  If there is no more
+   document, `NULL' is returned.  Because the object of the return value is opened with the
+   function `est_doc_new', it should be closed with the function `est_doc_delete' if it is no
+   longer in use. */
+ESTDOC *est_node_iter_next(ESTNODE *node);
+
+
 /* Add a header to a node connection object.
    `node' specifies a node connection object.
    `name' specifies the name of a header.
@@ -467,7 +601,13 @@ void est_noderes_add_doc(ESTNODERES *nres, CBMAP *attrs, char *snippet);
    `snippetp' specifies the pointer to a variable to which reference of the snippet string is
    assigned.  The region should be released with the function `free'.
    The return value is true if success, else it is false. */
-int est_noderes_shift(ESTNODERES *nres, CBMAP **attrp, char **snippetp);
+int est_noderes_shift_doc(ESTNODERES *nres, CBMAP **attrp, char **snippetp);
+
+
+/* Get the modification date of a node result object.
+   `nres' specifies a node result object.
+   The return value is the modification date of a node result object. */
+time_t est_noderes_mdate(ESTNODERES *nres);
 
 
 
